#include <libusb-1.0/libusb.h>
#include <string.h>
#include <time.h>
#include <stdio.h>
//#include <cassert>
//#include <cstdio>

#include "hid.h"
#include "iostream"//
using namespace std;

libusb_device_handle *dev_handle; //a device handle
libusb_context *ctx = NULL; //a libusb session

int hid_write(unsigned char *buf, int size)
{
	//int i;
	int actual,r;
	if(dev_handle == NULL)
	  return -1;
	//r = libusb_bulk_transfer(dev_handle, (LIBUSB_TRANSFER_TYPE_INTERRUPT | LIBUSB_ENDPOINT_OUT), buf, size, &actual, 0); //my device's out endpoint was 2, found with trial- the device had 2 endpoints: 2 and 129
	r = libusb_interrupt_transfer(dev_handle, 0x02, buf, size, &actual, 0);
	if(r == 0 && actual == size) //we wrote the 4 bytes successfully
	{
	    //cout<<"Writing Successful!"<<endl;
		return actual;
	}
	else
	{
	    //cout<<"Write Error"<<endl;
		return -2;
	}
}

static const int CONTROL_REQUEST_TYPE_IN = LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE;
static const int CONTROL_REQUEST_TYPE_OUT = LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE;

static const int HID_GET_REPORT = 0x01;
static const int HID_SET_REPORT = 0x09;
static const int HID_REPORT_TYPE_INPUT = 0x01;
static const int HID_REPORT_TYPE_OUTPUT = 0x02;
static const int HID_REPORT_TYPE_FEATURE = 0x03;

int hid_write_control(unsigned char *buf, int size)
{
	//int i;
	//int actual;
	int r;
	if(dev_handle == NULL)
	  return -1;
	
	r = libusb_control_transfer(
			dev_handle,
			CONTROL_REQUEST_TYPE_OUT ,
			HID_SET_REPORT,
			(HID_REPORT_TYPE_FEATURE<<8)|0x00,
			0,
			buf,
			size,
			0);

	if(r >= 0) //we wrote the 4 bytes successfully
	{
	    //cout<<"Writing Successful!"<<endl;
		return r;
	}
	else
	{
	    //cout<<"Write Error"<<endl;
		return -2;
	}
}

int hid_read(unsigned char *buf, int TimeOut)
{
	//int i;
	int actual,r;
	if(dev_handle == NULL)
	  return -1;
	//r = libusb_bulk_transfer(dev_handle, 0x81, buf, 19, &actual, TimeOut); //my device's out endpoint was 2, found with trial- the device had 2 endpoints: 2 and 129
	r = libusb_interrupt_transfer(dev_handle, 0x81, buf, 19, &actual, TimeOut);
	if(r < 0)
	    return r;

	return actual;
}

int hid_open(int VID, int PID)
{
	libusb_device **devs; //pointer to pointer of device, used to retrieve a list of devices

	int r; //for return values
	ssize_t cnt; //holding number of devices in list
	r = libusb_init(&ctx); //initialize the library for the session we just declared
	if(r < 0) {
		//cout<<"Init Error "<<r<<endl; //there was an error
		return -1;
	}
	libusb_set_debug(ctx, 3); //set verbosity level to 3, as suggested in the documentation
 
	cnt = libusb_get_device_list(ctx, &devs); //get the list of devices
	if(cnt < 0) {
		//cout<<"Get Device Error"<<endl; //there was an error
		return -2;
	}
	//cout<<cnt<<" Devices in list."<<endl;
 


	dev_handle = libusb_open_device_with_vid_pid(ctx, VID, PID); //these are vendorID and productID I found for my usb device
	if(dev_handle == NULL)
	{
		//cout<<"Cannot open device"<<endl;
		return -3;
	}
	else
		//cout<<"Device Opened"<<endl;
	libusb_free_device_list(devs, 1); //free the list, unref the devices in it
 
	//int actual; //used to find out how many bytes were written
	if(libusb_kernel_driver_active(dev_handle, 0) == 1) { //find out if kernel driver is attached
		;//cout<<"Kernel Driver Active"<<endl;
		if(libusb_detach_kernel_driver(dev_handle, 0) == 0) {//detach it
			;//cout<<"Kernel Driver Detached!"<<endl;
		}
	}
	r = libusb_claim_interface(dev_handle, 0); //claim interface 0 (the first) of device (mine had jsut 1)
	if(r < 0) {
		//cout<<"Cannot Claim Interface"<<endl;
		return -4;
	}
	//cout<<"Claimed Interface"<<endl;
	return r;
}

int hid_close()
{
	int r;
	r = libusb_release_interface(dev_handle, 0); //release the claimed interface
	if(r!=0) {
		//cout<<"Cannot Release Interface"<<endl;
		return 1;
	}
	//cout<<"Released Interface"<<endl;
 
	libusb_close(dev_handle); //close the device we opened
	libusb_exit(ctx); //needs to be called to end the
	return r;
 
}


int hid_open(int VID, int PID, uint16_t versionNo)
{
	libusb_device **devs; //pointer to pointer of device, used to retrieve a list of devices

	int r; //for return values
	ssize_t cnt; //holding number of devices in list
	r = libusb_init(&ctx); //initialize the library for the session we just declared
	if(r < 0) {
		//cout<<"Init Error "<<r<<endl; //there was an error
		return -1;
	}
	libusb_set_debug(ctx, 3); //set verbosity level to 3, as suggested in the documentation
 
	cnt = libusb_get_device_list(ctx, &devs); //get the list of devices
	if(cnt < 0) {
		//cout<<"Get Device Error"<<endl; //there was an error
		return -2;
	}
	//cout<<cnt<<" Devices in list."<<endl;


	
	for (ssize_t idx = 0; idx < cnt; ++idx) {
		libusb_device *device = devs[idx];
		libusb_device_descriptor desc = {0};

		r = libusb_get_device_descriptor(device, &desc);
		//cout<<"Vendor: "<<desc.idVendor<<" Product: "<<desc.idProduct<<" No: "<<desc.bcdDevice<<endl;
		if(desc.idVendor==VID && desc.idProduct==PID && (desc.bcdDevice==versionNo || versionNo==512)) {
			r = libusb_open(device, &dev_handle);
			break;
		}
	}

	if(dev_handle == NULL)
	{
		//cout<<"Cannot open device"<<endl;
		return -3;
	}
	else
		//cout<<"Device Opened"<<endl;
	libusb_free_device_list(devs, 1); //free the list, unref the devices in it
 
	//int actual; //used to find out how many bytes were written
	if(libusb_kernel_driver_active(dev_handle, 0) == 1) { //find out if kernel driver is attached
		;//cout<<"Kernel Driver Active"<<endl;
		if(libusb_detach_kernel_driver(dev_handle, 0) == 0) {//detach it
			;//cout<<"Kernel Driver Detached!"<<endl;
		}
	}
	r = libusb_claim_interface(dev_handle, 0); //claim interface 0 (the first) of device (mine had jsut 1)
	if(r < 0) {
		//cout<<"Cannot Claim Interface"<<endl;
		return -4;
	}
	//cout<<"Claimed Interface"<<endl;
	return r;

}


int FindAllHIDs(int VID, int PID, std::list<unsigned short> *irekList) {
	libusb_device **devs; //pointer to pointer of device, used to retrieve a list of devices

	int r; //for return values
	ssize_t cnt; //holding number of devices in list
	r = libusb_init(&ctx); //initialize the library for the session we just declared
	if(r < 0) {
		//cout<<"Init Error "<<r<<endl; //there was an error
		return -1;
	}
	libusb_set_debug(ctx, 3); //set verbosity level to 3, as suggested in the documentation
 
	cnt = libusb_get_device_list(ctx, &devs); //get the list of devices
	if(cnt < 0) {
		//cout<<"Get Device Error"<<endl; //there was an error
		return -2;
	}
	//cout<<cnt<<" Devices in list."<<endl;
	
	for (ssize_t idx = 0; idx < cnt; ++idx) {
		libusb_device *device = devs[idx];
		libusb_device_descriptor desc = {0};

		r = libusb_get_device_descriptor(device, &desc);
		if(desc.idVendor==VID && desc.idProduct==PID) {
			//cout<<"Vendor: "<<desc.idVendor<<" Product: "<<desc.idProduct<<" No: "<<desc.bcdDevice<<endl;
			irekList->push_back(desc.bcdDevice);
		}
	}
	return 0;
}
