#include "hid.c"
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <sys/stat.h>
#include "irek.h"

int VID=0x0925;
int PID=0x7001;

using namespace std;

// Wyjscie z trybu bootloadera
int ExitHIDboot() {
	int hb;
	int ret = 0;
	for(int i=10; i>0;) {		
		hb = hid_open(0x16c0, 0x05df);
		if (hb < 0) {
			//cout<<"Nie znaleziono HIDBoot!"<<endl;
			return ret;
		}
		else {
			cout<<"Polaczono z nadajnikiem w trybie HIDBoot"<<endl;		
			unsigned char FR[6];
			memset(FR, 0, sizeof(FR));
			hb = hid_write_control(FR, sizeof(FR));
			if (hb >= 0) {
				cout<<"Wyjscie z trybu bootloadera."<<endl;
				hid_close();
				ret = 1;
				cout<<"Czekaj 0,5s ..."<<endl;
				usleep(500000);
			}
			else {			
				hid_close();
				cout<<"Powtorna proba przelaczenia. Czekaj 0,2s ..."<<endl;	
				usleep(200000);
				i--;
			}
		}
	}
	cout<<"Blad zapisu danych do nadajnika w trybie boot!"<<endl;
	return -1;
}
// Polaczenie z nadajnikiem IRek
int ConnectIRek(unsigned short versionNo) {
	int r;
	for(int i=5; i>=0; i--) {
		r = hid_open(VID, PID, versionNo);
		if (r < 0) {
			cout<<"Oczekiwanie na polaczenie nadajnika w trybie nadawania. Pozostalo sekund: "<<i<<endl;
			sleep(1);
		}
		else {
			//cout<<"Polaczono z Nadajnikiem IRek T/2"<<endl;
			return 0;
		}
	}
	cout<<"Nie znaleziono nadajnika! Przerwano dzialanie programu."<<endl;
	return -5;
}

int FindAll(list<unsigned short> *irekList) {
	FindAllHIDs(VID, PID, irekList);
	if(irekList->size()) {
		irekList->sort();
		return 0;
	}
	else {
		cout<<"Nie znaleziono nadajnikow IRek. Przerwano dzialanie programu."<<endl;
		return -5;
	}
}

int SendCommand(char *filePath, unsigned int waitMS) {
	int r;
	unsigned int daneWyjsciowe[CALA_RAMKA];
	
	if(WszytajRamke(daneWyjsciowe, filePath) >= 0) { 
		//cout<<"Wczytano ramke danych z pliku"<<endl;
	}
	else {
		cout<<"Blad pliku fir! Przerwano dzialanie programu."<<endl;
		return -2;
	}	

	unsigned char	FeatureReport[REPORT_COUNT];
	memset(FeatureReport, 0, sizeof(FeatureReport));

	FeatureReport[0] = 1 << WYS_RAMKI | 1 << NADANIE_RAMKI; // komenda
	FeatureReport[1] = daneWyjsciowe[0] - 1; // liczba powtorzen
        FeatureReport[2] = daneWyjsciowe[1] % 256; // odstep pomierzy powtorzeniami (mlodszy bajt)
        FeatureReport[3] = (daneWyjsciowe[1] >> 8) % 256; // odstep pomierzy powtorzeniami (starszy bajt)

	int ByteNumber = 4;
	int i=0;
	for (i=2; i<CALA_RAMKA; i++) {
	    FeatureReport[ByteNumber++] = daneWyjsciowe[i] % 256;
	    FeatureReport[ByteNumber++] = (daneWyjsciowe[i] >> 8) % 256;
	}

	r = hid_write_control(FeatureReport, sizeof(FeatureReport));
	if (r >= 0) {
		//cout<<"Wyslano dane do nadajnika"<<endl;
	}
	else {
		cout<<"Blad zapisu danych do nadajnika!"<<endl;
	}
	hid_close();

	//cout<<"Zamkniecie polaczenia z nadajnikiem."<<endl;
	if(waitMS) {
		cout<<"Czekaj... "<<waitMS<<"ms"<<endl;
		usleep(waitMS*1000);
	}
	return 0;
}

int SendCommandToAll(char *filePath, unsigned int waitMS) {
	list<unsigned short> irekList;
	if(FindAll(&irekList) >= 0) {
	
		list<unsigned short>::iterator it;
		for (it=irekList.begin(); it!=irekList.end(); ++it) {
			if(ConnectIRek(*it) >= 0) {
				int r = SendCommand(filePath, 0);
				if(r >= 0) {
					cout<<"IRek T/2/"<<*it-512<<" -> Nadawanie... przerwa "<<waitMS<<"ms"<<endl;
					usleep(waitMS*1000);
				}
				else return r;
			}
			
		}
		return 0;
	}
	return -5;
}

int WszytajRamke(unsigned int *tab, char *filePath) {
	
	if(FileExist(filePath) > 0) {
		string line;
		ifstream firFile(filePath);
		if(firFile.is_open()) {
			for(int i=0; i<CALA_RAMKA; i++) {
				getline(firFile,line);
				tab[i] = atoi(line.c_str());
			}
		firFile.close();
		}
		else {
			cout<<"Blad otwarcia pliku!"<<endl;
			return -1;
		}
	}
	else {
		cout<<"Plik nie istnieje!"<<endl;
		return -1;
	}	
		
	return 0;
}

int FileExist(char *name) {
  struct stat   buffer;
  return (stat(name, &buffer) == 0);
}

